@@ -14,9 +14,12 @@
 # limitations under the License.
 
 import copy
+
 import testtools
 
 from openstack.cloud import exc
+from openstack.network.v2 import port as _port
+from openstack.network.v2 import router as _router
 from openstack.tests.unit import base
 
 
@@ -38,7 +41,12 @@ class TestRouter(base.TestCase):
         'id': router_id,
         'name': router_name,
         'project_id': u'861808a93da0484ea1767967c4df8a23',
-        'routes': [],
+        'routes': [
+            {
+                "destination": "179.24.1.0/24",
+                "nexthop": "172.24.3.99"
+            }
+        ],
         'status': u'ACTIVE',
         'tenant_id': u'861808a93da0484ea1767967c4df8a23'
     }
@@ -61,25 +69,52 @@ class TestRouter(base.TestCase):
         "name": "Router Availability Zone"
     }
 
-    enabled_neutron_extensions = [router_availability_zone_extension]
+    router_extraroute_extension = {
+        "alias": "extraroute",
+        "updated": "2015-01-01T10:00:00-00:00",
+        "description": "extra routes extension for router.",
+        "links": [],
+        "name": "Extra Routes"
+    }
+
+    enabled_neutron_extensions = [
+        router_availability_zone_extension,
+        router_extraroute_extension]
+
+    def _compare_routers(self, exp, real):
+        self.assertDictEqual(
+            _router.Router(**exp).to_dict(computed=False),
+            real.to_dict(computed=False))
 
     def test_get_router(self):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public',
+                     append=['v2.0', 'routers', self.router_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'routers'],
+                     qs_elements=['name=%s' % self.router_name]),
                  json={'routers': [self.mock_router_rep]})
         ])
         r = self.cloud.get_router(self.router_name)
         self.assertIsNotNone(r)
-        self.assertDictEqual(self.mock_router_rep, r)
+        self._compare_routers(self.mock_router_rep, r)
         self.assert_calls()
 
     def test_get_router_not_found(self):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public',
+                     append=['v2.0', 'routers', 'mickey']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'routers'],
+                     qs_elements=['name=mickey']),
                  json={'routers': []})
         ])
         r = self.cloud.get_router('mickey')
@@ -90,7 +125,7 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public', append=['v2.0', 'routers']),
                  json={'router': self.mock_router_rep},
                  validate=dict(
                      json={'router': {
@@ -99,7 +134,8 @@ class TestRouter(base.TestCase):
         ])
         new_router = self.cloud.create_router(name=self.router_name,
                                               admin_state_up=True)
-        self.assertDictEqual(self.mock_router_rep, new_router)
+
+        self._compare_routers(self.mock_router_rep, new_router)
         self.assert_calls()
 
     def test_create_router_specific_tenant(self):
@@ -110,13 +146,13 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public', append=['v2.0', 'routers']),
                  json={'router': mock_router_rep},
                  validate=dict(
                      json={'router': {
                          'name': self.router_name,
                          'admin_state_up': True,
-                         'tenant_id': new_router_tenant_id}}))
+                         'project_id': new_router_tenant_id}}))
         ])
 
         self.cloud.create_router(self.router_name,
@@ -127,11 +163,11 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'extensions.json']),
+                     'network', 'public', append=['v2.0', 'extensions']),
                  json={'extensions': self.enabled_neutron_extensions}),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public', append=['v2.0', 'routers']),
                  json={'router': self.mock_router_rep},
                  validate=dict(
                      json={'router': {
@@ -149,7 +185,7 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public', append=['v2.0', 'routers']),
                  json={'router': self.mock_router_rep},
                  validate=dict(
                      json={'router': {
@@ -165,7 +201,7 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public', append=['v2.0', 'routers']),
                  json={'router': self.mock_router_rep},
                  validate=dict(
                      json={'router': {
@@ -182,7 +218,7 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public', append=['v2.0', 'routers']),
                  json={'router': self.mock_router_rep},
                  validate=dict(
                      json={'router': {
@@ -210,7 +246,7 @@ class TestRouter(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public',
                      append=['v2.0', 'routers', self.router_id,
-                             'add_router_interface.json']),
+                             'add_router_interface']),
                  json={'port': self.mock_router_interface_rep},
                  validate=dict(
                      json={'subnet_id': self.subnet_id}))
@@ -225,7 +261,7 @@ class TestRouter(base.TestCase):
                  uri=self.get_mock_url(
                      'network', 'public',
                      append=['v2.0', 'routers', self.router_id,
-                             'remove_router_interface.json']),
+                             'remove_router_interface']),
                  json={'port': self.mock_router_interface_rep},
                  validate=dict(
                      json={'subnet_id': self.subnet_id}))
@@ -240,37 +276,55 @@ class TestRouter(base.TestCase):
 
     def test_update_router(self):
         new_router_name = "mickey"
+        new_routes = []
         expected_router_rep = copy.copy(self.mock_router_rep)
         expected_router_rep['name'] = new_router_name
+        expected_router_rep['routes'] = new_routes
+        # validate_calls() asserts that these requests are done in order,
+        # but the extensions call is only called if a non-None value is
+        # passed in 'routes'
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
-                 json={'routers': [self.mock_router_rep]}),
+                     'network', 'public', append=['v2.0', 'extensions']),
+                 json={'extensions': self.enabled_neutron_extensions}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'routers',
+                                                  self.router_id]),
+                 json=self.mock_router_rep),
             dict(method='PUT',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'routers', '%s.json' % self.router_id]),
+                     append=['v2.0', 'routers', self.router_id]),
                  json={'router': expected_router_rep},
                  validate=dict(
                      json={'router': {
-                         'name': new_router_name}}))
+                         'name': new_router_name,
+                         'routes': new_routes}}))
         ])
         new_router = self.cloud.update_router(
-            self.router_id, name=new_router_name)
-        self.assertDictEqual(expected_router_rep, new_router)
+            self.router_id, name=new_router_name, routes=new_routes)
+
+        self._compare_routers(expected_router_rep, new_router)
         self.assert_calls()
 
     def test_delete_router(self):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
+                     'network', 'public',
+                     append=['v2.0', 'routers', self.router_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'routers'],
+                     qs_elements=['name=%s' % self.router_name]),
                  json={'routers': [self.mock_router_rep]}),
             dict(method='DELETE',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'routers', '%s.json' % self.router_id]),
+                     append=['v2.0', 'routers', self.router_id]),
                  json={})
         ])
         self.assertTrue(self.cloud.delete_router(self.router_name))
@@ -280,8 +334,14 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
-                 json={'routers': []}),
+                     'network', 'public',
+                     append=['v2.0', 'routers', self.router_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'routers'],
+                     qs_elements=['name=%s' % self.router_name]),
+                 json={'routers': []})
         ])
         self.assertFalse(self.cloud.delete_router(self.router_name))
         self.assert_calls()
@@ -292,35 +352,24 @@ class TestRouter(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
-                 json={'routers': [router1, router2]}),
+                     'network', 'public',
+                     append=['v2.0', 'routers', 'mickey']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'routers'],
+                     qs_elements=['name=mickey']),
+                 json={'routers': [router1, router2]})
         ])
         self.assertRaises(exc.OpenStackCloudException,
                           self.cloud.delete_router,
                           'mickey')
         self.assert_calls()
 
-    def test_delete_router_multiple_using_id(self):
-        router1 = dict(id='123', name='mickey')
-        router2 = dict(id='456', name='mickey')
-        self.register_uris([
-            dict(method='GET',
-                 uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'routers.json']),
-                 json={'routers': [router1, router2]}),
-            dict(method='DELETE',
-                 uri=self.get_mock_url(
-                     'network', 'public',
-                     append=['v2.0', 'routers', '123.json']),
-                 json={})
-        ])
-        self.assertTrue(self.cloud.delete_router("123"))
-        self.assert_calls()
-
     def _get_mock_dict(self, owner, json):
         return dict(method='GET',
                     uri=self.get_mock_url(
-                        'network', 'public', append=['v2.0', 'ports.json'],
+                        'network', 'public', append=['v2.0', 'ports'],
                         qs_elements=["device_id=%s" % self.router_id,
                                      "device_owner=network:%s" % owner]),
                     json=json)
@@ -364,16 +413,20 @@ class TestRouter(base.TestCase):
         for port_type in ['router_interface',
                           'router_interface_distributed',
                           'ha_router_replicated_interface']:
-            ports = {}
             if port_type == device_owner:
                 ports = {'ports': [internal_port]}
+            else:
+                ports = {'ports': []}
             mock_uris.append(self._get_mock_dict(port_type, ports))
         mock_uris.append(self._get_mock_dict('router_gateway',
                                              {'ports': [external_port]}))
 
         self.register_uris(mock_uris)
         ret = self.cloud.list_router_interfaces(router, interface_type)
-        self.assertEqual(expected_result, ret)
+        self.assertEqual(
+            [_port.Port(**i).to_dict(computed=False) for i in expected_result],
+            [i.to_dict(computed=False) for i in ret]
+        )
         self.assert_calls()
 
     router = {
