@@ -1,7 +1,7 @@
 # Copyright 2017 OVH SAS
 # All Rights Reserved.
 #
-# Licensed under the Apache License, Version 2.0 (the 'License');
+# Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
@@ -14,9 +14,11 @@
 # limitations under the License.
 
 import copy
+
 import testtools
 
 from openstack.cloud import exc
+from openstack.network.v2 import subnet as _subnet
 from openstack.tests.unit import base
 
 
@@ -26,6 +28,8 @@ class TestSubnet(base.TestCase):
     subnet_name = 'subnet_name'
     subnet_id = '1f1696eb-7f47-47f6-835c-4889bff88604'
     subnet_cidr = '192.168.199.0/24'
+    subnetpool_cidr = '172.16.0.0/28'
+    prefix_length = 28
 
     mock_network_rep = {
         'id': '881d1bb7-a663-44c0-8f9f-ee2765b74486',
@@ -34,8 +38,8 @@ class TestSubnet(base.TestCase):
 
     mock_subnet_rep = {
         'allocation_pools': [{
-            'start': u'192.168.199.2',
-            'end': u'192.168.199.254'
+            'start': '192.168.199.2',
+            'end': '192.168.199.254'
         }],
         'cidr': subnet_cidr,
         'created_at': '2017-04-24T20:22:23Z',
@@ -57,16 +61,34 @@ class TestSubnet(base.TestCase):
         'tags': []
     }
 
+    mock_subnetpool_rep = {
+        'id': 'f49a1319-423a-4ee6-ba54-1d95a4f6cc68',
+        'prefixes': [
+            '172.16.0.0/16'
+        ]
+    }
+
+    def _compare_subnets(self, exp, real):
+        self.assertDictEqual(
+            _subnet.Subnet(**exp).to_dict(computed=False),
+            real.to_dict(computed=False))
+
     def test_get_subnet(self):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', self.subnet_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'subnets'],
+                     qs_elements=['name=%s' % self.subnet_name]),
                  json={'subnets': [self.mock_subnet_rep]})
         ])
         r = self.cloud.get_subnet(self.subnet_name)
         self.assertIsNotNone(r)
-        self.assertDictEqual(self.mock_subnet_rep, r)
+        self._compare_subnets(self.mock_subnet_rep, r)
         self.assert_calls()
 
     def test_get_subnet_by_id(self):
@@ -80,7 +102,7 @@ class TestSubnet(base.TestCase):
         ])
         r = self.cloud.get_subnet_by_id(self.subnet_id)
         self.assertIsNotNone(r)
-        self.assertDictEqual(self.mock_subnet_rep, r)
+        self._compare_subnets(self.mock_subnet_rep, r)
         self.assert_calls()
 
     def test_create_subnet(self):
@@ -94,11 +116,18 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
                  json={'networks': [self.mock_network_rep]}),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public', append=['v2.0', 'subnets']),
                  json={'subnet': mock_subnet_rep},
                  validate=dict(
                      json={'subnet': {
@@ -114,7 +143,7 @@ class TestSubnet(base.TestCase):
                                           allocation_pools=pool,
                                           dns_nameservers=dns,
                                           host_routes=routes)
-        self.assertDictEqual(mock_subnet_rep, subnet)
+        self._compare_subnets(mock_subnet_rep, subnet)
         self.assert_calls()
 
     def test_create_subnet_string_ip_version(self):
@@ -122,11 +151,18 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
                  json={'networks': [self.mock_network_rep]}),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public', append=['v2.0', 'subnets']),
                  json={'subnet': self.mock_subnet_rep},
                  validate=dict(
                      json={'subnet': {
@@ -137,7 +173,7 @@ class TestSubnet(base.TestCase):
         ])
         subnet = self.cloud.create_subnet(
             self.network_name, self.subnet_cidr, ip_version='4')
-        self.assertDictEqual(self.mock_subnet_rep, subnet)
+        self._compare_subnets(self.mock_subnet_rep, subnet)
         self.assert_calls()
 
     def test_create_subnet_bad_ip_version(self):
@@ -145,8 +181,15 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
-                 json={'networks': [self.mock_network_rep]})
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
+                 json={'networks': [self.mock_network_rep]}),
         ])
         with testtools.ExpectedException(
                 exc.OpenStackCloudException,
@@ -166,11 +209,18 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
                  json={'networks': [self.mock_network_rep]}),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public', append=['v2.0', 'subnets']),
                  json={'subnet': mock_subnet_rep},
                  validate=dict(
                      json={'subnet': {
@@ -186,7 +236,7 @@ class TestSubnet(base.TestCase):
                                           allocation_pools=pool,
                                           dns_nameservers=dns,
                                           disable_gateway_ip=True)
-        self.assertDictEqual(mock_subnet_rep, subnet)
+        self._compare_subnets(mock_subnet_rep, subnet)
         self.assert_calls()
 
     def test_create_subnet_with_gateway_ip(self):
@@ -200,11 +250,18 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
                  json={'networks': [self.mock_network_rep]}),
             dict(method='POST',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public', append=['v2.0', 'subnets']),
                  json={'subnet': mock_subnet_rep},
                  validate=dict(
                      json={'subnet': {
@@ -220,14 +277,21 @@ class TestSubnet(base.TestCase):
                                           allocation_pools=pool,
                                           dns_nameservers=dns,
                                           gateway_ip=gateway)
-        self.assertDictEqual(mock_subnet_rep, subnet)
+        self._compare_subnets(mock_subnet_rep, subnet)
         self.assert_calls()
 
     def test_create_subnet_conflict_gw_ops(self):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
+                     'network', 'public',
+                     append=['v2.0', 'networks', 'kooky']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=kooky']),
                  json={'networks': [self.mock_network_rep]})
         ])
         gateway = '192.168.200.3'
@@ -241,8 +305,15 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
-                 json={'networks': [self.mock_network_rep]})
+                     'network', 'public',
+                     append=['v2.0', 'networks', 'duck']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=duck']),
+                 json={'networks': [self.mock_network_rep]}),
         ])
         self.assertRaises(exc.OpenStackCloudException,
                           self.cloud.create_subnet,
@@ -255,24 +326,92 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'networks.json']),
-                 json={'networks': [net1, net2]})
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
+                 json={'networks': [net1, net2]}),
         ])
         self.assertRaises(exc.OpenStackCloudException,
                           self.cloud.create_subnet,
                           self.network_name, self.subnet_cidr)
         self.assert_calls()
 
+    def test_create_subnet_from_subnetpool_with_prefixlen(self):
+        pool = [{'start': '172.16.0.2', 'end': '172.16.0.15'}]
+        id = '143296eb-7f47-4755-835c-488123475604'
+        gateway = '172.16.0.1'
+        dns = ['8.8.8.8']
+        routes = [{"destination": "0.0.0.0/0", "nexthop": "123.456.78.9"}]
+        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
+        mock_subnet_rep['allocation_pools'] = pool
+        mock_subnet_rep['dns_nameservers'] = dns
+        mock_subnet_rep['host_routes'] = routes
+        mock_subnet_rep['gateway_ip'] = gateway
+        mock_subnet_rep['subnetpool_id'] = self.mock_subnetpool_rep['id']
+        mock_subnet_rep['cidr'] = self.subnetpool_cidr
+        mock_subnet_rep['id'] = id
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks', self.network_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public',
+                     append=['v2.0', 'networks'],
+                     qs_elements=['name=%s' % self.network_name]),
+                 json={'networks': [self.mock_network_rep]}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'subnets']),
+                 json={'subnet': mock_subnet_rep},
+                 validate=dict(
+                     json={'subnet': {
+                         'enable_dhcp': False,
+                         'ip_version': 4,
+                         'network_id': self.mock_network_rep['id'],
+                         'allocation_pools': pool,
+                         'dns_nameservers': dns,
+                         'use_default_subnetpool': True,
+                         'prefixlen': self.prefix_length,
+                         'host_routes': routes}}))
+        ])
+        subnet = self.cloud.create_subnet(self.network_name,
+                                          allocation_pools=pool,
+                                          dns_nameservers=dns,
+                                          use_default_subnetpool=True,
+                                          prefixlen=self.prefix_length,
+                                          host_routes=routes)
+        mock_subnet_rep.update(
+            {
+                'prefixlen': self.prefix_length,
+                'use_default_subnetpool': True
+            })
+        self._compare_subnets(mock_subnet_rep, subnet)
+        self.assert_calls()
+
     def test_delete_subnet(self):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', self.subnet_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'subnets'],
+                     qs_elements=['name=%s' % self.subnet_name]),
                  json={'subnets': [self.mock_subnet_rep]}),
             dict(method='DELETE',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
+                     append=['v2.0', 'subnets', self.subnet_id]),
                  json={})
         ])
         self.assertTrue(self.cloud.delete_subnet(self.subnet_name))
@@ -282,7 +421,13 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', 'goofy']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'subnets'],
+                     qs_elements=['name=goofy']),
                  json={'subnets': []})
         ])
         self.assertFalse(self.cloud.delete_subnet('goofy'))
@@ -294,7 +439,13 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', self.subnet_name]),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'network', 'public', append=['v2.0', 'subnets'],
+                     qs_elements=['name=%s' % self.subnet_name]),
                  json={'subnets': [subnet1, subnet2]})
         ])
         self.assertRaises(exc.OpenStackCloudException,
@@ -302,18 +453,18 @@ class TestSubnet(base.TestCase):
                           self.subnet_name)
         self.assert_calls()
 
-    def test_delete_subnet_multiple_using_id(self):
+    def test_delete_subnet_using_id(self):
         subnet1 = dict(id='123', name=self.subnet_name)
-        subnet2 = dict(id='456', name=self.subnet_name)
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
-                 json={'subnets': [subnet1, subnet2]}),
+                     'network', 'public', append=['v2.0', 'subnets',
+                                                  subnet1['id']]),
+                 json=subnet1),
             dict(method='DELETE',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'subnets', '%s.json' % subnet1['id']]),
+                     append=['v2.0', 'subnets', subnet1['id']]),
                  json={})
         ])
         self.assertTrue(self.cloud.delete_subnet(subnet1['id']))
@@ -325,18 +476,19 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
-                 json={'subnets': [self.mock_subnet_rep]}),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', self.subnet_id]),
+                 json=self.mock_subnet_rep),
             dict(method='PUT',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
+                     append=['v2.0', 'subnets', self.subnet_id]),
                  json={'subnet': expected_subnet},
                  validate=dict(
                      json={'subnet': {'name': 'goofy'}}))
         ])
         subnet = self.cloud.update_subnet(self.subnet_id, subnet_name='goofy')
-        self.assertDictEqual(expected_subnet, subnet)
+        self._compare_subnets(expected_subnet, subnet)
         self.assert_calls()
 
     def test_update_subnet_gateway_ip(self):
@@ -346,18 +498,19 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
-                 json={'subnets': [self.mock_subnet_rep]}),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', self.subnet_id]),
+                 json=self.mock_subnet_rep),
             dict(method='PUT',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
+                     append=['v2.0', 'subnets', self.subnet_id]),
                  json={'subnet': expected_subnet},
                  validate=dict(
                      json={'subnet': {'gateway_ip': gateway}}))
         ])
         subnet = self.cloud.update_subnet(self.subnet_id, gateway_ip=gateway)
-        self.assertDictEqual(expected_subnet, subnet)
+        self._compare_subnets(expected_subnet, subnet)
         self.assert_calls()
 
     def test_update_subnet_disable_gateway_ip(self):
@@ -366,19 +519,20 @@ class TestSubnet(base.TestCase):
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
-                     'network', 'public', append=['v2.0', 'subnets.json']),
-                 json={'subnets': [self.mock_subnet_rep]}),
+                     'network', 'public',
+                     append=['v2.0', 'subnets', self.subnet_id]),
+                 json=self.mock_subnet_rep),
             dict(method='PUT',
                  uri=self.get_mock_url(
                      'network', 'public',
-                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
+                     append=['v2.0', 'subnets', self.subnet_id]),
                  json={'subnet': expected_subnet},
                  validate=dict(
                      json={'subnet': {'gateway_ip': None}}))
         ])
         subnet = self.cloud.update_subnet(self.subnet_id,
                                           disable_gateway_ip=True)
-        self.assertDictEqual(expected_subnet, subnet)
+        self._compare_subnets(expected_subnet, subnet)
         self.assert_calls()
 
     def test_update_subnet_conflict_gw_ops(self):
