@@ -15,6 +15,7 @@ import uuid
 
 from openstack import exceptions
 from openstack.network.v2 import _proxy
+from openstack.network.v2 import address_group
 from openstack.network.v2 import address_scope
 from openstack.network.v2 import agent
 from openstack.network.v2 import auto_allocated_topology
@@ -26,12 +27,16 @@ from openstack.network.v2 import firewall_rule
 from openstack.network.v2 import flavor
 from openstack.network.v2 import floating_ip
 from openstack.network.v2 import health_monitor
+from openstack.network.v2 import ikepolicy
 from openstack.network.v2 import ipsec_site_connection
 from openstack.network.v2 import l3_conntrack_helper
 from openstack.network.v2 import listener
 from openstack.network.v2 import load_balancer
+from openstack.network.v2 import local_ip
+from openstack.network.v2 import local_ip_association
 from openstack.network.v2 import metering_label
 from openstack.network.v2 import metering_label_rule
+from openstack.network.v2 import ndp_proxy
 from openstack.network.v2 import network
 from openstack.network.v2 import network_ip_availability
 from openstack.network.v2 import network_segment_range
@@ -42,6 +47,7 @@ from openstack.network.v2 import port_forwarding
 from openstack.network.v2 import qos_bandwidth_limit_rule
 from openstack.network.v2 import qos_dscp_marking_rule
 from openstack.network.v2 import qos_minimum_bandwidth_rule
+from openstack.network.v2 import qos_minimum_packet_rate_rule
 from openstack.network.v2 import qos_policy
 from openstack.network.v2 import qos_rule_type
 from openstack.network.v2 import quota
@@ -66,6 +72,7 @@ AGENT_ID = 'agent-id-' + uuid.uuid4().hex
 ROUTER_ID = 'router-id-' + uuid.uuid4().hex
 FIP_ID = 'fip-id-' + uuid.uuid4().hex
 CT_HELPER_ID = 'ct-helper-id-' + uuid.uuid4().hex
+LOCAL_IP_ID = 'lip-id-' + uuid.uuid4().hex
 
 
 class TestNetworkProxy(test_proxy_base.TestProxyBase):
@@ -106,6 +113,63 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
             expected_kwargs=expected_kwargs,
             mock_method=mock_method)
 
+
+class TestNetworkAddressGroup(TestNetworkProxy):
+    def test_address_group_create_attrs(self):
+        self.verify_create(self.proxy.create_address_group,
+                           address_group.AddressGroup)
+
+    def test_address_group_delete(self):
+        self.verify_delete(self.proxy.delete_address_group,
+                           address_group.AddressGroup,
+                           False)
+
+    def test_address_group_delete_ignore(self):
+        self.verify_delete(self.proxy.delete_address_group,
+                           address_group.AddressGroup,
+                           True)
+
+    def test_address_group_find(self):
+        self.verify_find(self.proxy.find_address_group,
+                         address_group.AddressGroup)
+
+    def test_address_group_get(self):
+        self.verify_get(self.proxy.get_address_group,
+                        address_group.AddressGroup)
+
+    def test_address_groups(self):
+        self.verify_list(self.proxy.address_groups,
+                         address_group.AddressGroup)
+
+    def test_address_group_update(self):
+        self.verify_update(self.proxy.update_address_group,
+                           address_group.AddressGroup)
+
+    @mock.patch('openstack.network.v2._proxy.Proxy.'
+                'add_addresses_to_address_group')
+    def test_add_addresses_to_address_group(self, add_addresses):
+        data = mock.sentinel
+
+        self.proxy.add_addresses_to_address_group(address_group.AddressGroup,
+                                                  data)
+
+        add_addresses.assert_called_once_with(address_group.AddressGroup,
+                                              data)
+
+    @mock.patch('openstack.network.v2._proxy.Proxy.'
+                'remove_addresses_from_address_group')
+    def test_remove_addresses_from_address_group(self, remove_addresses):
+        data = mock.sentinel
+
+        self.proxy.remove_addresses_from_address_group(
+            address_group.AddressGroup,
+            data)
+
+        remove_addresses.assert_called_once_with(address_group.AddressGroup,
+                                                 data)
+
+
+class TestNetworkAddressScope(TestNetworkProxy):
     def test_address_scope_create_attrs(self):
         self.verify_create(self.proxy.create_address_scope,
                            address_scope.AddressScope)
@@ -136,6 +200,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_address_scope,
                            address_scope.AddressScope)
 
+
+class TestNetworkAgent(TestNetworkProxy):
     def test_agent_delete(self):
         self.verify_delete(self.proxy.delete_agent, agent.Agent, True)
 
@@ -148,6 +214,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
     def test_agent_update(self):
         self.verify_update(self.proxy.update_agent, agent.Agent)
 
+
+class TestNetworkAvailability(TestNetworkProxy):
     def test_availability_zones(self):
         self.verify_list(
             self.proxy.availability_zones,
@@ -169,6 +237,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
             expected_kwargs={'network_id': NETWORK_ID}
         )
 
+
+class TestNetworkExtension(TestNetworkProxy):
     def test_extension_find(self):
         self.verify_find(self.proxy.find_extension, extension.Extension)
 
@@ -212,6 +282,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                            expected_kwargs={'x': 1, 'y': 2, 'z': 3,
                                             'if_revision': 42})
 
+
+class TestNetworkHealthMonitor(TestNetworkProxy):
     def test_health_monitor_create_attrs(self):
         self.verify_create(self.proxy.create_health_monitor,
                            health_monitor.HealthMonitor)
@@ -240,6 +312,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_health_monitor,
                            health_monitor.HealthMonitor)
 
+
+class TestNetworkSiteConnection(TestNetworkProxy):
     def test_ipsec_site_connection_create_attrs(self):
         self.verify_create(self.proxy.create_vpn_ipsec_site_connection,
                            ipsec_site_connection.IPSecSiteConnection)
@@ -268,6 +342,38 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_vpn_ipsec_site_connection,
                            ipsec_site_connection.IPSecSiteConnection)
 
+
+class TestNetworkIkePolicy(TestNetworkProxy):
+    def test_ikepolicy_create_attrs(self):
+        self.verify_create(self.proxy.create_vpn_ikepolicy,
+                           ikepolicy.IkePolicy)
+
+    def test_ikepolicy_delete(self):
+        self.verify_delete(self.proxy.delete_vpn_ikepolicy,
+                           ikepolicy.IkePolicy, False)
+
+    def test_ikepolicy_delete_ignore(self):
+        self.verify_delete(self.proxy.delete_vpn_ikepolicy,
+                           ikepolicy.IkePolicy, True)
+
+    def test_ikepolicy_find(self):
+        self.verify_find(self.proxy.find_vpn_ikepolicy,
+                         ikepolicy.IkePolicy)
+
+    def test_ikepolicy_get(self):
+        self.verify_get(self.proxy.get_vpn_ikepolicy,
+                        ikepolicy.IkePolicy)
+
+    def test_ikepolicies(self):
+        self.verify_list(self.proxy.vpn_ikepolicies,
+                         ikepolicy.IkePolicy)
+
+    def test_ikepolicy_update(self):
+        self.verify_update(self.proxy.update_vpn_ikepolicy,
+                           ikepolicy.IkePolicy)
+
+
+class TestNetworkListener(TestNetworkProxy):
     def test_listener_create_attrs(self):
         self.verify_create(self.proxy.create_listener, listener.Listener)
 
@@ -291,6 +397,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
     def test_listener_update(self):
         self.verify_update(self.proxy.update_listener, listener.Listener)
 
+
+class TestNetworkLoadBalancer(TestNetworkProxy):
     def test_load_balancer_create_attrs(self):
         self.verify_create(self.proxy.create_load_balancer,
                            load_balancer.LoadBalancer)
@@ -319,6 +427,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_load_balancer,
                            load_balancer.LoadBalancer)
 
+
+class TestNetworkMeteringLabel(TestNetworkProxy):
     def test_metering_label_create_attrs(self):
         self.verify_create(self.proxy.create_metering_label,
                            metering_label.MeteringLabel)
@@ -375,6 +485,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_metering_label_rule,
                            metering_label_rule.MeteringLabelRule)
 
+
+class TestNetworkNetwork(TestNetworkProxy):
     def test_network_create_attrs(self):
         self.verify_create(self.proxy.create_network, network.Network)
 
@@ -421,6 +533,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                            expected_kwargs={'x': 1, 'y': 2, 'z': 3,
                                             'if_revision': 42})
 
+
+class TestNetworkFlavor(TestNetworkProxy):
     def test_flavor_create_attrs(self):
         self.verify_create(self.proxy.create_flavor, flavor.Flavor)
 
@@ -439,6 +553,106 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
     def test_flavors(self):
         self.verify_list(self.proxy.flavors, flavor.Flavor)
 
+
+class TestNetworkLocalIp(TestNetworkProxy):
+    def test_local_ip_create_attrs(self):
+        self.verify_create(self.proxy.create_local_ip, local_ip.LocalIP)
+
+    def test_local_ip_delete(self):
+        self.verify_delete(self.proxy.delete_local_ip, local_ip.LocalIP,
+                           False, expected_kwargs={'if_revision': None})
+
+    def test_local_ip_delete_ignore(self):
+        self.verify_delete(self.proxy.delete_local_ip, local_ip.LocalIP,
+                           True, expected_kwargs={'if_revision': None})
+
+    def test_local_ip_delete_if_revision(self):
+        self.verify_delete(self.proxy.delete_local_ip, local_ip.LocalIP,
+                           True, method_kwargs={'if_revision': 42},
+                           expected_kwargs={'if_revision': 42})
+
+    def test_local_ip_find(self):
+        self.verify_find(self.proxy.find_local_ip, local_ip.LocalIP)
+
+    def test_local_ip_get(self):
+        self.verify_get(self.proxy.get_local_ip, local_ip.LocalIP)
+
+    def test_local_ips(self):
+        self.verify_list(self.proxy.local_ips, local_ip.LocalIP)
+
+    def test_local_ip_update(self):
+        self.verify_update(self.proxy.update_local_ip, local_ip.LocalIP,
+                           expected_kwargs={'x': 1, 'y': 2, 'z': 3,
+                                            'if_revision': None})
+
+    def test_local_ip_update_if_revision(self):
+        self.verify_update(self.proxy.update_local_ip, local_ip.LocalIP,
+                           method_kwargs={'x': 1, 'y': 2, 'z': 3,
+                                          'if_revision': 42},
+                           expected_kwargs={'x': 1, 'y': 2, 'z': 3,
+                                            'if_revision': 42})
+
+
+class TestNetworkLocalIpAssociation(TestNetworkProxy):
+    def test_local_ip_association_create_attrs(self):
+        self.verify_create(self.proxy.create_local_ip_association,
+                           local_ip_association.LocalIPAssociation,
+                           method_kwargs={'local_ip': LOCAL_IP_ID},
+                           expected_kwargs={'local_ip_id': LOCAL_IP_ID})
+
+    def test_local_ip_association_delete(self):
+        self.verify_delete(
+            self.proxy.delete_local_ip_association,
+            local_ip_association.LocalIPAssociation,
+            ignore_missing=False,
+            method_args=[LOCAL_IP_ID, "resource_or_id"],
+            expected_args=["resource_or_id"],
+            expected_kwargs={'if_revision': None,
+                             'local_ip_id': LOCAL_IP_ID})
+
+    def test_local_ip_association_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_local_ip_association,
+            local_ip_association.LocalIPAssociation,
+            ignore_missing=True,
+            method_args=[LOCAL_IP_ID, "resource_or_id"],
+            expected_args=["resource_or_id"],
+            expected_kwargs={'if_revision': None,
+                             'local_ip_id': LOCAL_IP_ID})
+
+    def test_local_ip_association_find(self):
+        lip = local_ip.LocalIP.new(id=LOCAL_IP_ID)
+
+        self._verify(
+            'openstack.proxy.Proxy._find',
+            self.proxy.find_local_ip_association,
+            method_args=['local_ip_association_id', lip],
+            expected_args=[
+                local_ip_association.LocalIPAssociation,
+                'local_ip_association_id'],
+            expected_kwargs={
+                'ignore_missing': True, 'local_ip_id': LOCAL_IP_ID})
+
+    def test_local_ip_association_get(self):
+        lip = local_ip.LocalIP.new(id=LOCAL_IP_ID)
+
+        self._verify(
+            'openstack.proxy.Proxy._get',
+            self.proxy.get_local_ip_association,
+            method_args=['local_ip_association_id', lip],
+            expected_args=[
+                local_ip_association.LocalIPAssociation,
+                'local_ip_association_id'],
+            expected_kwargs={'local_ip_id': LOCAL_IP_ID})
+
+    def test_local_ip_associations(self):
+        self.verify_list(self.proxy.local_ip_associations,
+                         local_ip_association.LocalIPAssociation,
+                         method_kwargs={'local_ip': LOCAL_IP_ID},
+                         expected_kwargs={'local_ip_id': LOCAL_IP_ID})
+
+
+class TestNetworkServiceProfile(TestNetworkProxy):
     def test_service_profile_create_attrs(self):
         self.verify_create(self.proxy.create_service_profile,
                            service_profile.ServiceProfile)
@@ -463,6 +677,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_service_profile,
                            service_profile.ServiceProfile)
 
+
+class TestNetworkIpAvailability(TestNetworkProxy):
     def test_network_ip_availability_find(self):
         self.verify_find(self.proxy.find_network_ip_availability,
                          network_ip_availability.NetworkIPAvailability)
@@ -481,6 +697,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                            method_kwargs={"pool": "test_id"},
                            expected_kwargs={"pool_id": "test_id"})
 
+
+class TestNetworkPoolMember(TestNetworkProxy):
     def test_pool_member_delete(self):
         self.verify_delete(
             self.proxy.delete_pool_member,
@@ -528,6 +746,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
             expected_args=[pool_member.PoolMember, "MEMBER"],
             expected_kwargs={"pool_id": "POOL"})
 
+
+class TestNetworkPool(TestNetworkProxy):
     def test_pool_create_attrs(self):
         self.verify_create(self.proxy.create_pool, pool.Pool)
 
@@ -594,6 +814,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
 
         bc.assert_called_once_with(port.Port, data)
 
+
+class TestNetworkQosBandwidth(TestNetworkProxy):
     def test_qos_bandwidth_limit_rule_create_attrs(self):
         self.verify_create(
             self.proxy.create_qos_bandwidth_limit_rule,
@@ -657,6 +879,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                 'rule_id'],
             expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'})
 
+
+class TestNetworkQosDscpMarking(TestNetworkProxy):
     def test_qos_dscp_marking_rule_create_attrs(self):
         self.verify_create(
             self.proxy.create_qos_dscp_marking_rule,
@@ -719,6 +943,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                 'rule_id'],
             expected_kwargs={'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'})
 
+
+class TestNetworkQosMinimumBandwidth(TestNetworkProxy):
     def test_qos_minimum_bandwidth_rule_create_attrs(self):
         self.verify_create(
             self.proxy.create_qos_minimum_bandwidth_rule,
@@ -783,29 +1009,74 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
             expected_kwargs={
                 'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'})
 
-    def test_qos_policy_create_attrs(self):
-        self.verify_create(self.proxy.create_qos_policy, qos_policy.QoSPolicy)
 
-    def test_qos_policy_delete(self):
-        self.verify_delete(self.proxy.delete_qos_policy, qos_policy.QoSPolicy,
-                           False)
+class TestNetworkQosMinimumPacketRate(TestNetworkProxy):
+    def test_qos_minimum_packet_rate_rule_create_attrs(self):
+        self.verify_create(
+            self.proxy.create_qos_minimum_packet_rate_rule,
+            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+            method_kwargs={'qos_policy': QOS_POLICY_ID},
+            expected_kwargs={'qos_policy_id': QOS_POLICY_ID})
 
-    def test_qos_policy_delete_ignore(self):
-        self.verify_delete(self.proxy.delete_qos_policy, qos_policy.QoSPolicy,
-                           True)
+    def test_qos_minimum_packet_rate_rule_delete(self):
+        self.verify_delete(
+            self.proxy.delete_qos_minimum_packet_rate_rule,
+            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+            ignore_missing=False,
+            method_args=["resource_or_id", QOS_POLICY_ID],
+            expected_args=["resource_or_id"],
+            expected_kwargs={'qos_policy_id': QOS_POLICY_ID})
 
-    def test_qos_policy_find(self):
-        self.verify_find(self.proxy.find_qos_policy, qos_policy.QoSPolicy)
+    def test_qos_minimum_packet_rate_rule_delete_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_qos_minimum_packet_rate_rule,
+            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+            ignore_missing=True,
+            method_args=["resource_or_id", QOS_POLICY_ID],
+            expected_args=["resource_or_id"],
+            expected_kwargs={'qos_policy_id': QOS_POLICY_ID})
 
-    def test_qos_policy_get(self):
-        self.verify_get(self.proxy.get_qos_policy, qos_policy.QoSPolicy)
+    def test_qos_minimum_packet_rate_rule_find(self):
+        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
+        self._verify(
+            'openstack.proxy.Proxy._find',
+            self.proxy.find_qos_minimum_packet_rate_rule,
+            method_args=['rule_id', policy],
+            expected_args=[
+                qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+                'rule_id'],
+            expected_kwargs={
+                'ignore_missing': True, 'qos_policy_id': QOS_POLICY_ID})
 
-    def test_qos_policies(self):
-        self.verify_list(self.proxy.qos_policies, qos_policy.QoSPolicy)
+    def test_qos_minimum_packet_rate_rule_get(self):
+        self.verify_get(
+            self.proxy.get_qos_minimum_packet_rate_rule,
+            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+            method_kwargs={'qos_policy': QOS_POLICY_ID},
+            expected_kwargs={'qos_policy_id': QOS_POLICY_ID})
 
-    def test_qos_policy_update(self):
-        self.verify_update(self.proxy.update_qos_policy, qos_policy.QoSPolicy)
+    def test_qos_minimum_packet_rate_rules(self):
+        self.verify_list(
+            self.proxy.qos_minimum_packet_rate_rules,
+            qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+            method_kwargs={'qos_policy': QOS_POLICY_ID},
+            expected_kwargs={'qos_policy_id': QOS_POLICY_ID})
 
+    def test_qos_minimum_packet_rate_rule_update(self):
+        policy = qos_policy.QoSPolicy.new(id=QOS_POLICY_ID)
+        self._verify(
+            'openstack.network.v2._proxy.Proxy._update',
+            self.proxy.update_qos_minimum_packet_rate_rule,
+            method_args=['rule_id', policy],
+            method_kwargs={'foo': 'bar'},
+            expected_args=[
+                qos_minimum_packet_rate_rule.QoSMinimumPacketRateRule,
+                'rule_id'],
+            expected_kwargs={
+                'qos_policy_id': QOS_POLICY_ID, 'foo': 'bar'})
+
+
+class TestNetworkQosRuleType(TestNetworkProxy):
     def test_qos_rule_type_find(self):
         self.verify_find(self.proxy.find_qos_rule_type,
                          qos_rule_type.QoSRuleType)
@@ -817,6 +1088,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
     def test_qos_rule_types(self):
         self.verify_list(self.proxy.qos_rule_types, qos_rule_type.QoSRuleType)
 
+
+class TestNetworkQuota(TestNetworkProxy):
     def test_quota_delete(self):
         self.verify_delete(self.proxy.delete_quota, quota.Quota, False)
 
@@ -859,6 +1132,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
     def test_quota_update(self):
         self.verify_update(self.proxy.update_quota, quota.Quota)
 
+
+class TestNetworkRbacPolicy(TestNetworkProxy):
     def test_rbac_policy_create_attrs(self):
         self.verify_create(self.proxy.create_rbac_policy,
                            rbac_policy.RBACPolicy)
@@ -884,6 +1159,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_rbac_policy,
                            rbac_policy.RBACPolicy)
 
+
+class TestNetworkRouter(TestNetworkProxy):
     def test_router_create_attrs(self):
         self.verify_create(self.proxy.create_router, router.Router)
 
@@ -1063,6 +1340,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
             expected_kwargs={'agent_id': AGENT_ID},
         )
 
+
+class TestNetworkFirewallGroup(TestNetworkProxy):
     def test_firewall_group_create_attrs(self):
         self.verify_create(self.proxy.create_firewall_group,
                            firewall_group.FirewallGroup)
@@ -1091,6 +1370,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_firewall_group,
                            firewall_group.FirewallGroup)
 
+
+class TestNetworkPolicy(TestNetworkProxy):
     def test_firewall_policy_create_attrs(self):
         self.verify_create(self.proxy.create_firewall_policy,
                            firewall_policy.FirewallPolicy)
@@ -1119,6 +1400,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_firewall_policy,
                            firewall_policy.FirewallPolicy)
 
+
+class TestNetworkRule(TestNetworkProxy):
     def test_firewall_rule_create_attrs(self):
         self.verify_create(self.proxy.create_firewall_rule,
                            firewall_rule.FirewallRule)
@@ -1147,6 +1430,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_firewall_rule,
                            firewall_rule.FirewallRule)
 
+
+class TestNetworkNetworkSegment(TestNetworkProxy):
     def test_network_segment_range_create_attrs(self):
         self.verify_create(self.proxy.create_network_segment_range,
                            network_segment_range.NetworkSegmentRange)
@@ -1175,6 +1460,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_network_segment_range,
                            network_segment_range.NetworkSegmentRange)
 
+
+class TestNetworkSecurityGroup(TestNetworkProxy):
     def test_security_group_create_attrs(self):
         self.verify_create(self.proxy.create_security_group,
                            security_group.SecurityGroup)
@@ -1261,6 +1548,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
 
         bc.assert_called_once_with(security_group_rule.SecurityGroupRule, data)
 
+
+class TestNetworkSegment(TestNetworkProxy):
     def test_segment_create_attrs(self):
         self.verify_create(self.proxy.create_segment, segment.Segment)
 
@@ -1282,6 +1571,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
     def test_segment_update(self):
         self.verify_update(self.proxy.update_segment, segment.Segment)
 
+
+class TestNetworkSubnet(TestNetworkProxy):
     def test_subnet_create_attrs(self):
         self.verify_create(self.proxy.create_subnet, subnet.Subnet)
 
@@ -1340,6 +1631,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_subnet_pool,
                            subnet_pool.SubnetPool)
 
+
+class TestNetworkVpnService(TestNetworkProxy):
     def test_vpn_service_create_attrs(self):
         self.verify_create(self.proxy.create_vpn_service,
                            vpn_service.VPNService)
@@ -1366,10 +1659,14 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
         self.verify_update(self.proxy.update_vpn_service,
                            vpn_service.VPNService)
 
+
+class TestNetworkServiceProvider(TestNetworkProxy):
     def test_service_provider(self):
         self.verify_list(self.proxy.service_providers,
                          service_provider.ServiceProvider)
 
+
+class TestNetworkAutoAllocatedTopology(TestNetworkProxy):
     def test_auto_allocated_topology_get(self):
         self.verify_get(self.proxy.get_auto_allocated_topology,
                         auto_allocated_topology.AutoAllocatedTopology)
@@ -1392,6 +1689,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                         expected_kwargs={"project": mock.sentinel.project_id,
                                          "requires_id": False})
 
+
+class TestNetworkTags(TestNetworkProxy):
     def test_set_tags(self):
         x_network = network.Network.new(id='NETWORK_ID')
         self._verify(
@@ -1409,6 +1708,8 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                           no_tag_resource, ['TAG1', 'TAG2'])
         self.assertEqual(0, mock_set_tags.call_count)
 
+
+class TestNetworkFloatingIp(TestNetworkProxy):
     def test_create_floating_ip_port_forwarding(self):
         self.verify_create(self.proxy.create_floating_ip_port_forwarding,
                            port_forwarding.PortForwarding,
@@ -1529,3 +1830,28 @@ class TestNetworkProxy(test_proxy_base.TestProxyBase):
                 l3_conntrack_helper.ConntrackHelper,
                 'conntrack_helper_id'],
             expected_kwargs={'router_id': ROUTER_ID, 'foo': 'bar'})
+
+
+class TestNetworkNDPProxy(TestNetworkProxy):
+    def test_ndp_proxy_create_attrs(self):
+        self.verify_create(self.proxy.create_ndp_proxy, ndp_proxy.NDPProxy)
+
+    def test_ndp_proxy_delete(self):
+        self.verify_delete(self.proxy.delete_ndp_proxy, ndp_proxy.NDPProxy,
+                           False)
+
+    def test_ndp_proxy_delete_ignore(self):
+        self.verify_delete(self.proxy.delete_ndp_proxy, ndp_proxy.NDPProxy,
+                           True)
+
+    def test_ndp_proxy_find(self):
+        self.verify_find(self.proxy.find_ndp_proxy, ndp_proxy.NDPProxy)
+
+    def test_ndp_proxy_get(self):
+        self.verify_get(self.proxy.get_ndp_proxy, ndp_proxy.NDPProxy)
+
+    def test_ndp_proxies(self):
+        self.verify_list(self.proxy.ndp_proxies, ndp_proxy.NDPProxy)
+
+    def test_ndp_proxy_update(self):
+        self.verify_update(self.proxy.update_ndp_proxy, ndp_proxy.NDPProxy)
